# NOT PRODUCTION READY

# Operator - Sign PYTH-SPL Transaction

Solana blockchain has a time sensitive aspect associated to validity of standard transactions. The `blockhash` which is used as part of a transaction expires in 60-90 seconds. This introduces operational challenges to signing a transaction offline. As a result, this ceremony requires 3 operators, rather than the typical 2. It is essential for the operators to collaborate to quickly get the transaction data from the online computer to the offline, sign it, then get it back to the online machine before the `blockhash` validity period expires.

The online machine operator is only to operate the online machine, and not touch the offline machine, and the air-gapped machine operators should not touch the online machine. The operators must focus on their machine and their part of the process.

Typically, the online machine and the additional operator are not necessary as there is no time sensitivity to the transaction as only some blockchains have the requirement of using a `blockhash` from a previous block as part of a new valid transaction.

## Requirements

* 3 Operators

    * 2 primary operators will be operating the offline machine

        * Ensure both primary operators have their [Operator Keys](../../../../../../glossary.md#operator-key)

    * An additional operator is necessary for fetching and providing the transaction data and the latest SOL `blockhash` from a online computer and transmitting using an SD card to the 2 primary operators conducting the main ceremony

* Photographic tamper proofing evidence

    * Both operators should print photographic evidence from digital cameras which is stored in a PGP signed repository. The photographs should be of the top and underside of the vacuum sealed object.

        * The operators should verify the commit signatures of the photographs they are printing against a list of permitted PGP keys

            * TODO: where do we refer to permitted PGP keys

* Ensure location has [tamper proofing tools](../../../../../../tamper-evidence-methods.md#vacuum-sealed-bags-with-filler)

    * Vacuum sealer

    * Vacuum roll

    * Colored beads

* 4 SD cards (2 fresh, formatted as ext4, and 2 cards with prepared data)

    * 1 SD card for transferring transaction data from online to air-gapped machine

    * 1 SD card for storing tamper proofing evidence produced at the end of the ceremony

    * 1 SD card which has the shardfile and "trusted keys" for proposers and approvers, both signed by each operator using their operator key (TODO)

        * This should be write-locked and stored in tamper proofing along with air-gapped machine

            * TODO refactor for this to be alongside airgapped machine

    * 1 SD card with AirgapOS

* Digital camera (TODO selection)

* [Online machine](../../../../../../online-machine-provisioning.md) used for fetching transaction data

## Procedure

1. Enter the designated location with the 3 operators and all required equipment

2. Lock access to the location - there should be no inflow or outflow of people during the ceremony

3. Retrieve sealed laptop and polaroid from locked storage

### Unsealing Tamper Proofing

{{ #include ../../../../../../tamper-evidence-methods.md:vsbwf-procedure-unsealing}}

### Secure Boot Procedure

{{ #include ../../../../../../secure-boot-sequence.md:content}}

0. Load well known PGP keys of proposer and approver, and sign them using operator keys (NOT IMPLEMENTED)

1. Insert SD card with shardfile

2. `keyfork recover shard --daemon`

3. Await prompt and plug in first Operator Key

4. Tap the key (may have to tap multiple times)

5. Await prompt and plug in second Operator Key

6. Tap the key

7. Run `keyfork

8. As a last step, run the `icepick` command which is awaiting the transaction payload

    * TODO add command

### Obtain Transaction Request

1. Turn on online machine

2. Get transaction request(s)

    * TODO define means (could just be email?)

3. Run `icepick workflow sol-get-blockhash-and-broadcast` command

    * Wait for prompt and plug in fresh SD card

    * Await completion message before removing SD card

    * This command will set the computer into "awaiting mode", which will broadcast the signed transaction from the SD card once it's plugged back in

5. Unplug the SD card and pass it to the air-gapped machine operators

### Sign Transaction

1. Use `icepick` to generate the transaction payload:

    * `icepick workflow sol-transfer-token`

        * Wait for SD card prompt and plug in SD card with signed transaction payload

2. Wait for the screen to display the transaction information. (NOT IMPLEMENTED)

    * In the background:

        * The transaction is constructed

        * Signatures of tx data are verified against well known keys which were loaded by operators into local GPG keychain and signed by operators (NOT IMPLEMENTED)

3. If any issues are detected with data you will be prompted and should initiate [incident response (todo)](todo)

4. Wait for the "completed" message

5. Unplug and give the SD card back to the online machine operator

### Broadcast Transaction

* Online machine operator takes the SD card to online machine and plugs it in

* The still running process from running the command to create the transaction in [Obtain Transaction Request](#obtain-transaction-request) will broadcast the transaction automatically

* Await the "completed" message

### Finalization

* Shut down online machine

* Shut down the air gapped machine

#### Sealing

{{ #include ../../../../../../tamper-evidence-methods.md:vsbwf-procedure-sealing}}

